//Source file for Game rendering.

#include <header.h>
#include <global.h>
#include <gameglobal.h>
#include <game.h>
#include <ascii.h>
#include <ascrender.h>
#include <player.h>
#include <train.h>
#include <log.h>

//Uninitialize Game Renderer
void quit_game_renderer(){

	log_printf("Quitting Game Renderer");
}

//Initialize Game Renderer
int init_game_renderer(){

	log_printf("Initializing Game Renderer");
	
	return 0;
}

//Draw Tracks
void draw_tracks(BITMAP *bitmap, int x, int y, int ntracks, int bgcolor, int flags){
	int i;
		
	for(i=0;i<ntracks;++i){
		rectfill(bitmap,x,y,x+(aatrack->width*xmul),y+(aatrack->height*ymul),bgcolor);
		draw_asciiart(bitmap, aatrack, x, y);
		x+=(aatrack->width)*xmul;
	}		
}

//Draw Smoke
void draw_smoke(BITMAP *bitmap, int x, int y){	
	int val=(rand()%21)+160;
	char smokedata[6]={",.;:\'\""};	

	//Random Colorize
	aasmoke->r=val;  aasmoke->g=val; aasmoke->b=val; 

	//Randomize
	*(aasmoke->data+((rand()%aasmoke->height)*aasmoke->width)+(rand()%aasmoke->width))=smokedata[rand()%6];

	//Draw
	draw_asciiart(bitmap, aasmoke, x, y-(aasmoke->height*ymul));	
}

//Draw Trains
void draw_trains(BITMAP *bitmap, int x, int y, int ntrains, int *traindata, int bgcolor, int flags){
	int i;
	
	for(i=0;i<ntrains;++i){
		if(traindata[i]==SPR_COACH) {
			rectfill(bitmap,x,y,x+((aacoach->width-1)*xmul),y+((aacoach->height-1)*ymul),bgcolor);
			draw_asciiart(bitmap, aacoach, x, y);
			x+=(aacoach->width-1)*xmul;
			draw_asciiart(bitmap, aaconnector, x, y);
			x+=(aaconnector->width*xmul);
		}
		if(traindata[i]==SPR_COACH2) {
			rectfill(bitmap,x,y+((aacoach2->height-4)*ymul),x+((aacoach2->width-1)*xmul),y+((aacoach2->height-1)*ymul),bgcolor);
			draw_asciiart(bitmap, aacoach2, x, y);
			x+=(aacoach2->width-1)*xmul;
			draw_asciiart(bitmap, aaconnector, x, y);
			x+=(aaconnector->width*xmul);
		}
		if(traindata[i]==SPR_COACH3) {
			rectfill(bitmap,x,y,x+((aacoach3->width-1)*xmul),y+((aacoach3->height-1)*ymul),bgcolor);
			draw_asciiart(bitmap, aacoach3, x, y);
			x+=(aacoach3->width-1)*xmul;
			draw_asciiart(bitmap, aaconnector, x, y);
			x+=(aaconnector->width*xmul);
		}
		if(traindata[i]==SPR_ENGINE) {
			rectfill(bitmap,x,y,x+(6*xmul),y+(2*ymul),bgcolor);
			rectfill(bitmap,x,y+(2*ymul),x+((aacoach->width-3)*xmul),y+((aacoach->height-1)*ymul),bgcolor);
			draw_asciiart(bitmap, aaengine, x, y);
			draw_smoke(bitmap, x+(aaengine->width-3)*xmul, y+ymul);
			x+=(aaengine->width)*xmul;
		}
	}
		
}

//Render Player
void draw_player(BITMAP *bitmap, int x, int y, int state, int flags){
	int sx=x,sy=y;

	if(playerstate==PLAYER_JUMP) {
		sy-=ymul*(aaplayerj->height-1);
		draw_asciiart(bitmap, aaplayerj, sx, sy);
	}
	else if(playerstate==PLAYER_CROUCH) {
		sy-=ymul*(aaplayer->height-1);
		draw_asciiart(bitmap, aaplayerc, sx, sy);		
	}
	else if(playerstate==PLAYER_STAND) {
		sy-=ymul*(aaplayer->height-1);
		draw_asciiart(bitmap, aaplayer, sx, sy);		
	}
	
}

//Draw Timer Digits
void render_number(BITMAP *bitmap, int x, int y, char num){
	int tnum=0;
	if(num==':') tnum=10;
	else if(num>='0'&&num<='9') tnum=num-'0';
	else return;
	asciiart_bitmap_blit(aanumbers, bitmap, 0, tnum*6, aanumbers->width-1, (tnum+1)*6, x, y);
}

//Draw Timer
void render_timer(BITMAP *bitmap, int x, int y, int flags){
	int min,sec;
	int xoffset;		
	char timer_buffer[MAXSTR],*ptr;	

	//Calculate
	sec=(availtime)/lps;
	min=sec/60; sec%=60;
	
	*timer_buffer=0;
	sprintf(timer_buffer,"%02d:%02d",min,sec);

	ptr=timer_buffer;
	xoffset=x;	
	while(*ptr) {
		render_number(bitmap,xoffset,y,*(ptr++));
		xoffset+=xmul*aanumbers->width;
	}
}

//Render Game Stuff
void render_gamestuff(BITMAP *bitmap, int camx, int camy, int flags){
	int ntracks=0;
	int trxoff=0;

	trxoff=-(camx%am->tilewidth);
	
	ntracks=bitmap->w/(aatrack->width*xmul);
	if(trxoff) ++ntracks;

	//Draw Tracks
	draw_tracks(bitmap,trxoff,trainy-camy+(3*ymul),ntracks,makecol(am->r,am->g,am->b),flags);

	//Draw Trains
	draw_trains(bitmap,trainx-camx,trainy-camy,am->ntrains,am->traindata,makecol(am->r,am->g,am->b),flags);

	//Draw Player
	draw_player(bitmap,playerx-camx,playery-camy,playerstate,flags);

	//Draw Timer
	render_timer(bitmap,120,0,flags);
}