//Source file for Game States.

#include <header.h>
#include <global.h>
#include <game.h>
#include <gamestate.h>

int state_terminated=0,state_switched=0;
int oldstate=GAME_USUAL,state=GAME_USUAL;

//Logic for States
void state_logic(){

	if(state_terminated) {
		if(state_terminated>0) {
			state_quit();
			terminate_loop();
			state_terminated=-1; //One Time Quit.
		}
		return;
	}

	if(state_switched) --state_switched;

	switch(state){
		case GAME_PLAY: game_play_logic(); break;
		case GAME_LOST: game_lost_logic(); break;
		case GAME_WIN: game_win_logic(); break;
		case GAME_MENU: game_menu_logic(); break;
		case GAME_CREDITS: game_credits_logic(); break;
		case GAME_HELP: game_help_logic(); break;
		default: usual_logic();
	}
}

//Input for States
void state_input(){

	if(state_switched) return;
	if(state_terminated) return;

	switch(state){
		case GAME_PLAY: game_play_input(); break;
		case GAME_LOST: game_lost_input(); break;
		case GAME_WIN: game_win_input(); break;
		case GAME_MENU: game_menu_input(); break;
		case GAME_CREDITS: game_credits_input(); break;
		case GAME_HELP: game_help_input(); break;
		default: usual_input();
	}
}

//Render for States
void state_render(){

	if(state_switched) return;
	if(state_terminated) return;

	//Clear Buffer
	clear_bitmap(dbuffer);

	switch(state){
		case GAME_PLAY: game_play_render(); break;
		case GAME_LOST: game_lost_render(); break;
		case GAME_WIN: game_win_render(); break;
		case GAME_MENU: game_menu_render(); break;
		case GAME_CREDITS: game_credits_render(); break;
		case GAME_HELP: game_help_render(); break;
		default: usual_render();
	}
	
	//Common for All Games
	blit(dbuffer,screen,0,0,0,0,xres,yres);

}

//Quit for States
void state_quit(){
	switch(state){
		case GAME_PLAY: game_play_quit(); break;
		case GAME_LOST: game_lost_quit(); break;
		case GAME_WIN: game_win_quit(); break;
		case GAME_MENU: game_menu_quit(); break;
		case GAME_CREDITS: game_credits_quit(); break;
		case GAME_HELP: game_help_quit(); break;
		default: usual_quit();
	}
}

//Init for States
void state_init(){
	switch(state){
		case GAME_PLAY: game_play_init(); break;
		case GAME_LOST: game_lost_init(); break;
		case GAME_WIN: game_win_init(); break;
		case GAME_MENU: game_menu_init(); break;
		case GAME_CREDITS: game_credits_init(); break;
		case GAME_HELP: game_help_init(); break;
		default: usual_init();
	}
}


//Quit for States
void state_switch(int newstate){
	state_quit();
	oldstate=state;
	state=newstate;
	++state_switched;
	state_init();
}

//Quit State Loop
void terminate_state_loop(){
	++state_switched;	
	++state_terminated;
}